#
# aplicação que mostra limites do intervalo de confiança para uma distribuição binomial
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# License: Creative Commons: CC BY-SA
#

library(shiny)
resolution <- 0.1
tolerance <- 0.0001

server <- function(input, output) {
    # númeto de experimentos
    n <- reactive ({
        input$n
    })
    
    # número de sucessos na amostra
    sampleSuccess <- reactive ({
        input$sampleSuccess
    })
    
    # limite inferior sugerido do intervalo de confiança
    infLimit <- reactive ({
        input$infLimit
    })
    
    # limite superior sugerido do intervalo de confiança
    supLimit <- reactive ({
        input$supLimit
    })
    
    # metade do valor de alfa
    half.alfa <- reactive ({
        (1 - input$conflevel / 100) / 2
    })
    
    # probabilidade de sucesso na amostra
    prob <- reactive ({
        sampleSuccess() / n()
    })
    
    output$superiorLimit <- renderPlot({
        # mensagem de erro caso o número de sucessos seja maior que o número de experimentos
        if (sampleSuccess() > n()) {
            plot(c(0, 1), c(0, 1), type = "n", axes = FALSE, yaxt = 'n', xaxt = 'n', xlab = "", ylab = "")
            text(0, 0.5, "O número de sucessos não pode ser maior do que o número de testes", cex = 1.2, pos = 4)
        } else {
            # traça a distribuição binomial com probabilidade igual ao
            # produto do limite superior e número de experimentos
            ymax = dbinom(trunc(supLimit() * n()), size = n(), prob = supLimit())
            plot(c(0, n()), c(0, ymax), type = "n", xlab = "", ylab = "", main = "Limite Superior", ylim = c(0, ymax), xaxt = 'n')
            # calcula a probabilidade da binomial para valores abaixo ou igual ao número de sucessos na amostra
            p <- pbinom(sampleSuccess(), size = n(), prob = supLimit())
            # se p é igual a alfa/2, 
            # desenha os segmentos para valores iguais ou abaixo do número de sucessos em verde
            # caso contrário, desenha em vermelho.
            # Os segmentos para valores acima do número de sucessos 
            # são desenhados em preto
            color <- if (abs(p - half.alfa()) < tolerance) {
                rgb(0, 1, 0, 0.5)
            } else {
                rgb(1, 0, 0, 0.5)
            }
            mtext(paste("Binomial(", sprintf("%d", n()), ", ", sprintf("%.2f", supLimit()), ")", sep = ""), cex = 1.2)
            sequence <- c(0:sampleSuccess())
            for (x in sequence) {
                segments(x, 0, x, dbinom(x, size = n(), prob = supLimit()), col = color, lwd = 3)
            }
            sequence <- c(sampleSuccess() + 1:n())
            for (x in sequence) {
                segments(x, 0, x, dbinom(x, size = n(), prob = supLimit()), col = 'black', lwd = 3)
            }
            ypos <- 3 * par("usr")[4] / 4
            xpos <- par("usr")[2] / 6
            tick <- sampleSuccess()
            ticks <- c(seq(0, tick - 1, 1), seq(tick + 1, n(), 1))
            axis(1, at = ticks)
            par(cex = 1.5)
            # escreve o valor de alfa/2 na tela em (xpos, ypos)
            text(xpos,  ypos, paste('alfa/2 = ', sprintf('%.4f', p)), pos = 4)
            # coloca uma marca mais forte na posição igual ao número de sucessos na amostra
            axis(1, at = sampleSuccess(), lwd.ticks = 6)
        }
    })
    
    output$inferiorLimit <- renderPlot({
        # mensagem de erro caso o número de sucessos seja maior que o número de experimentos
        if (sampleSuccess() > n()) {
            plot(c(0, 1), c(0, 1), type = "n", axes = FALSE, yaxt = 'n', xaxt = 'n', xlab = "", ylab = "")
            text(0, 0.5, "O número de sucessos não pode ser maior do que o número de testes", cex = 1.2, pos = 4)
        } else {
            # traça a distribuição binomial com probabilidade igual ao
            # produto do limite superior e número de experimentos
            ymax = dbinom(trunc(infLimit() * n()), size = n(), prob = infLimit())
            plot(c(0, n()), c(0, ymax), type = "n", xlab = "", ylab = "", main = "Limite Inferior", ylim = c(0, ymax), xaxt = 'n')
            # calcula a probabilidade da binomial para valores acima ou igual ao número de sucessos na amostra
            p <- pbinom(sampleSuccess() - 1, size = n(), prob = infLimit(), lower.tail = FALSE)
            # se p é igual a alfa/2, 
            # desenha os segmentos para valores iguais ou acima do número de sucessos em verde
            # caso contrário, desenha em vermelho.
            # Os segmentos para valores abaixo do número de sucessos 
            # são desenhados em preto
            color <- if (abs(p - half.alfa()) < tolerance) {
                rgb(0, 1, 0, 0.5)
            } else {
                rgb(1, 0, 0, 0.5)
            }
            mtext(paste("Binomial(", sprintf("%d", n()), ", ", sprintf("%.2f", infLimit()), ")", sep = ""), cex = 1.2)
            sequence <- c(0:sampleSuccess() - 1)
            for (x in sequence) {
                segments(x, 0, x, dbinom(x, size = n(), prob = infLimit()), col = "black", lwd = 3)
            }
            sequence <- c(sampleSuccess():n())
            for (x in sequence) {
                segments(x, 0, x, dbinom(x, size = n(), prob = infLimit()), col = color, lwd = 3)
            }
            ypos <- 3 * par("usr")[4] / 4
            xpos <- 3 * par("usr")[2] / 4
            tick <- sampleSuccess()
            ticks <- c(seq(0, tick - 1, 1), seq(tick + 1, n(), 1))
            axis(1, at = ticks)
            par(cex = 1.5)
            # escreve o valor de alfa/2 na tela em (xpos, ypos)
            text(xpos,  ypos, paste("alfa/2 = ", sprintf("%.4f",  p)), pos = 4)
            # coloca uma marca mais forte na posição igual ao número de sucessos na amostra
            axis(1, at = sampleSuccess(), lwd.ticks = 6)
        }
    })
}


ui <- fluidPage(titlePanel(expression("Limites do intervalo de confiança")),
                
                sidebarLayout(
                    sidebarPanel(
                        sliderInput(
                            "conflevel", label = expression("Nível de Confiança (%)"),
                            min = 0, max = 100, step = 1, value = 95
                        ),
                        sliderInput(
                            "n", label = expression("Número de experimentos"),
                            min = 1, max = 50, value = 8
                        ),
                        sliderInput(
                            "sampleSuccess", label = expression("Número de sucessos"),
                            min = 0, max = 50, step = 1, value = 6
                        ),
                        numericInput(
                            "supLimit", label = "Limite superior", 
                            min = 0, max = 1, step = .01, value = .8
                        ),
                        numericInput(
                            "infLimit", label = "Limite Inferior",
                            min = 0, max = 1, step = .01, value = .6
                        )
                    ),
                    
                    mainPanel(fluidRow(
                        column(10, plotOutput("superiorLimit")),
                        column(10, plotOutput("inferiorLimit"))
                    ))
                ))

shinyApp(ui = ui, server = server)