#
# aplicação que mostra a distribuicao da variância amostral para diversos tamanhos amostrais e distribuições normais 
# da variável aleatória original
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(RcmdrMisc)

server <- function(input, output) {
  
  # Média da distribuição normal   
  meanNormal <- reactive ({
      input$mean
  })

  # Desvio padrão da distribuição normal   
  sdNormal <- reactive ({
      input$stdev
  })

  # número de amostras
  sampleCount <- reactive ({
    input$sampleCount
  })
  
  # Tamanho de cada amostra
  sampleSize <- reactive ({
    input$sampleSize
  })


  # número de observações
  npoints <- reactive({
      sampleSize() * sampleCount()
  })

  # número de classes no histograma 
  classes <- reactive({
    input$breaks
  })
  
  # Distribuição normal
  yNormal <- reactive ({
    rnorm(npoints(), mean=meanNormal(), sd=sdNormal())
  })

  # traça o gráfico da distribuição normal
  output$distribution <- renderPlot({
      inf_limit <- meanNormal() - 4 * sdNormal()
      sup_limit <- meanNormal() + 4 * sdNormal()
      curve(dnorm(x, meanNormal(), sdNormal()), xlim = c(inf_limit, sup_limit),
                  main = paste("N(", sprintf("%.1f", meanNormal()), ", ", sprintf("%.1f", sdNormal() ^ 2), ")", sep = ""),
                  xlab = "", ylab = "")
  })
  
  # plota o histograma da variância amostral x (n-1) / variância da distribuição original superposto à  distribuição Qui quadrado
  output$sampleVarianceHistogram <- renderPlot({
    # x é uma matriz de duas dimensões (tamanho da amostra, número de amostras)
    y <- matrix(yNormal(), nc = sampleCount())
    variancia = sdNormal()*sdNormal()
    gl <- sampleSize()-1
    #   valor mínimo do eixo x
    #   valor máximo do eixo x
    min_xaxis <- 0
    max_xaxis <- qchisq(0.9999, df=gl)
    # yvar é um vetor que contém a variância de cada amostra da matriz y
    yvar <- apply(y, 2, var)
    # valores de quiquadrado para cada amostra    
    qui <- (yvar * gl) / variancia
    h <- hist(qui, freq=FALSE, breaks = classes(), plot = FALSE)
    # máximo valor do histograma ou da distribuição t
    yHmax <- max(h$density)
    ymax <- max(yHmax, 0)
    limits <- par("usr")
    ypos <- par("usr")[4]/2 # obtém a metade do valor máximo do eixo y
    xpos <- min_xaxis + (max_xaxis - min_xaxis)/2
    par(cex=1.0)
    # valor de qui quadrado que delimita a área de 0.025 na cauda inferior
    cutoff1 <- qchisq(0.025, gl)
    # valor de qui quadrado que delimita a área de 0.025 na cauda superior
    cutoff2 <- qchisq(0.975, gl)
    x <- seq(min_xaxis, max_xaxis, length.out=500)
    # curva qui quadrado
    plotDistr(x, dchisq(x, gl), xlab="", ylab="", main="Distribuição Qui quadrado", xlim=c(min_xaxis, max_xaxis),
              region=list(c(min_xaxis, cutoff1), c(cutoff2, max_xaxis)), col=rgb(1, 0, 0,0.5), legend = FALSE, lwd = "2", add = TRUE)
    hist(qui, freq=FALSE, breaks = classes(), xlab = "", ylab=expression("Densidade de Frequência"), 
         xlim=c(min_xaxis, max_xaxis), ylim = c(0, ymax), add = TRUE)
    lines(x, dchisq(x, gl), type = "l", col = "blue", lwd = 2)
    ypos <- ymax*3/4 # 75% do valor máximo do eixo y
    xpos <- max_xaxis/3
    par(cex=1.2)
    legend(xpos, ypos, c(paste("X(0.025,", sprintf("%d", gl), ") = ", sprintf("%.2f", cutoff1)), paste("X(0.975,", sprintf("%d", gl), ") = ", sprintf("%.2f", cutoff2))),  fill=c("red"))
  })
}

ui <- fluidPage(
  
  titlePanel(expression("Distribuição t de Student")),
  
  sidebarLayout(
    sidebarPanel(
      sliderInput("mean", 
                      expression("Média da distribuição normal"), 
                      min = 0, max = 100, value = 10
      ),
      sliderInput("stdev", 
                      expression("Desvio padrão da distribuição normal"), 
                      min = 0, max = 50,
                      value = 1
      ),
      sliderInput("sampleSize", 
                      "Tamanho amostral", 
                      min = 5, max = 200,
                      value = 2
      ),
      sliderInput("sampleCount", 
                  expression("Número de amostras"), 
                  min = 1000, max = 100000,
                  value = 10000
      ),
      sliderInput("breaks", 
                  expression("Número de classes no histograma:"), 
                  min = 20, max = 200,
                  value = 20
      ),
      submitButton("Submeter")
    ),
    
    mainPanel(
      fluidRow(
        column(8, plotOutput("distribution")),
        column(8, plotOutput("sampleVarianceHistogram"))
      )
    )
  )
)

shinyApp(ui = ui, server = server)