#
# applicação que mostra a influência da especificidade sobre a probabilidade pós-teste
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA
#

library(shiny)
library(DT)

# Função que fornece o valor preditivo positivo
vpp <- function(prev, sens, spec) {
    (prev * sens) / (prev*sens + (1-spec) * (1 - prev))
}

# Função que fornece o valor preditivo negativo
vpn <- function(prev, sens, spec) {
    (1-prev) * spec  / (prev* (1 - sens) + spec * (1 - prev))
}

# Define server logic required to draw the curve
server <- function(input, output) {

#Parâmetros escolhidos pelo usuário: especificidade, sensibilidade
  specificity <- reactive ({
    input$specificity
  })
  
  sensitivity <- reactive ({
    input$sensitivity
  })

  pretestProb <- reactive({
    input$pretestProb
  })
  
  testTable <- reactive ({
    total = 100
    d = total * pretestProb()
    d_ = 100 - d
    vp = sensitivity() * d
    fn = d - vp
    vn = specificity() * d_
    fp = d_ - vn
    vpp = vp / (vp+fp)
    vpn = vn / (vn + fn)
    diseased <- paste(sprintf("%.2f", d))
    nonDiseased <- paste(sprintf("%.2f", d_))
    truePositive <- paste(sprintf("%.2f", vp))
    falseNegative <- paste(sprintf("%.2f", fn))
    trueNegative <- paste(sprintf("%.2f", vn))
    falsePositive <- paste(sprintf("%.2f", fp))
    vppText <- paste("VPP = ", sprintf("%.2f", vpp))
    vpnText <- paste("VPN = ", sprintf("%.2f", vpn))
    
    table <- matrix(nrow = 0, ncol = 4)
    line1 <- "T+"
    line2 <- "T-"
    line3 <- "Total"
    text1 <- c(line1, truePositive, falsePositive, vppText)
    table <- rbind(table, text1)
    text2 <- c(line2, falseNegative, trueNegative, vpnText )
    table <- rbind(table, text2)
    text3 <- c(line3, diseased, nonDiseased, total)
    table <- rbind(table, text3)
    colnames(table) <-
      c(" ", "D+", "D-", "Probabilidades")
    rownames(table) <- NULL
    table
  })

  output$tableComponents <-
#    renderTable({ testTable()}, align = 'r')
    DT::renderDataTable({testTable()}, options = list(
      autoWidth = TRUE, rownames = FALSE, dom = 't', 
      rowCallback = JS("function(r,d) {$(r).attr('height', '30px')}"),
      columnDefs = list(list(width = '10%', targets = 0:3), list(targets=0:3, class="dt-center"))
    ))
  
 # Gráfico do valor preditivo positivo
  output$vpp_curve <- renderPlot({
    curve(vpp(x, sensitivity(), specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", lwd = 2)
    # curvas de referência  
    curve(vpp(x, sensitivity(), .7), xlim=c(0,1), main="", xlab = "", ylab = "", col = "green", add = TRUE)
    curve(vpp(x, sensitivity(), .8), xlim=c(0,1), main="", xlab = "", ylab = "", col = "red", add = TRUE)
    curve(vpp(x, sensitivity(), .9), xlim=c(0,1), main="", xlab = "", ylab = "", col = "blue", add = TRUE)
    par(cex=1.5)
    title(xlab = "Probabilidade Pré-teste", ylab = "Valor Preditivo Positivo")
    legend(.5, .5, c("E = 0,7", "E = 0,8", "E = 0,9"), fill=c("green", "red", "blue"))
  })
  
  output$vpn_curve <- renderPlot({
      curve(vpn(x, sensitivity(), specificity()), xlim=c(0,1),main="", xlab = "", ylab = "", lwd = 2)
      # curvas de referência  
      curve(vpn(x, sensitivity(), .7), xlim=c(0,1), main="", xlab = "", ylab = "", col = "green", add = TRUE)
      curve(vpn(x, sensitivity(), .8), xlim=c(0,1), main="", xlab = "", ylab = "", col = "red", add = TRUE)
      curve(vpn(x, sensitivity(), .9), xlim=c(0,1), main="", xlab = "", ylab = "", col = "blue", add = TRUE)
      par(cex=1.5)
      title(xlab = "Probabilidade Pré-teste", ylab = "Valor Preditivo Negativo")
      legend(.5, .5, c("E = 0,7", "E = 0,8", "E = 0,9"), fill=c("green", "red", "blue"))
  })
  
}

ui <- fluidPage(
  
  titlePanel("Influência da Especificidade sobre a Probabilidade Pós-Teste"),
  
  sidebarLayout(
    sidebarPanel(style = "font-size: 20px; ",
                 tags$style("[type = 'number'] {font-size:20px;height:50px;}"),
          numericInput("pretestProb", 
                label = "Probabilidade pré-teste", 
                min = 0, max = 1, step = .05,
                value = 0.60
          ),
          numericInput("sensitivity", 
                 label = "Sensibilidade", 
                 min = 0, max = 1, step = .05,
                 value = 0.98
          ),
          tags$style(type = "text/css", "
              .irs-single {color:white; font-size: 20px; top: -10px}
          "),
          sliderInput("specificity", 
                 label = "Especificidade", 
                 min = 0, max = .9999,
                 value = 0.75
          )
   ),

    # Show plots
    mainPanel(
        fluidRow(
          column(8, align="center", div(dataTableOutput("tableComponents"), style = "font-size:150%")),
          column(8, plotOutput("vpp_curve")),
          column(8, plotOutput("vpn_curve"))
        )
    )
  )
)


shinyApp(ui = ui, server = server)