#  
#   Intervalo de Confiança 
#
#  Com reactiveValues para os parâmetros mais as amostras 
#  Agora tudo está sincronizado. O nível de confiança somente altera a largura dos IC
#
#  Linhas de referência
#  vários conjuntos de amostra simulateamente e botão para limpar a tela
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(stats)

server <- function(input, output) {

    # geração de num_amostras de tamanho n da distribuição normal com média e desvio
    # padrão selecionados pelo usuário
    rv <- reactiveValues(num_amostras = 20, n = 10, media_pop = 80, dp = 20, dados = NULL, cont_conj_amostras = 0)
        
    observeEvent(input$reamostrar, {  
            if ((rv$n != input$n) | (rv$dp != input$dp) | (rv$media_pop != input$media) |
                (rv$num_amostras != input$num_amostras))  {
              limparDados()
            }
            # número de amostras
            rv$num_amostras <- as.numeric(input$num_amostras)
            # tamanho de cada amostra
            rv$n <- input$n
            # média da população
            rv$media_pop <- input$media
            # desvio padrão
            rv$dp <- input$dp
            # contador do conjunto de amostras
            rv$cont_conj_amostras <-  rv$cont_conj_amostras + 1
            rv$dados <- cbind(rv$dados, matrix(rnorm(rv$n * rv$num_amostras, mean = rv$media_pop, sd = rv$dp), nc = rv$num_amostras))
    })
    
    observeEvent(input$reset, {
      limparDados()
    })       

  limparDados <- function() {
    rv$dados <- NULL
    rv$cont_conj_amostras <- 0
  }  
    
  output$ics <- renderPlot({
    # metade do valor de alfa  
    metade_alfa <- (100 - as.numeric(substring(input$confianca, 0, 2))) / 200
    # Erro padrão da média amostral   
    erro_padrao <- rv$dp / sqrt(rv$n)
    #   valor mínimo do eixo x
    min_x <- rv$media_pop - 4 * rv$dp
    #   valor máximo do eixo x
    max_x <- rv$media_pop + 4 * rv$dp
    # metade do valor de alfa   
    #  ponto da distribuição da média amostral cuja área à esquerda é alfa/2 
    posicao_max <- 210  # posição superior onde os ICs começarão a ser desenhados
    quant_inf <- rv$media_pop + qnorm(metade_alfa, mean = 0, sd = 1, lower.tail = TRUE) * erro_padrao
    #  ponto da distribuição da média amostral cuja área à direita é alfa/2 
    quant_sup <- rv$media_pop + qnorm(metade_alfa, mean = 0, sd = 1, lower.tail = FALSE) * erro_padrao
    par(cex = 1.5)
    plot(c(min_x, max_x), c(1, posicao_max), main = "Intervalos de confiança", type = "n", xlab = "", ylab = "", yaxt = 'n')
    if (input$linha_ref == TRUE) {
      # traça as linhas que delimitam áreas iguais a alfa/2 à esquerda e à direita da distribuição normal
      abline(v = quant_inf, col = "red", lwd = 2)
      abline(v = quant_sup, col = "red", lwd = 2)
      # traça a linha correspondente à média da distribuição normal
      abline(v = rv$media_pop, col = "black", lwd = 2)
    }
    if (is.null(rv$dados)) return()
    num_colunas = rv$num_amostras * rv$cont_conj_amostras
    if (num_colunas != ncol(rv$dados)) {
        rv$dados = NULL
        return
    }
    if (rv$num_amostras == 1) {
        passo <- 10
    } else {
        passo <- posicao_max / rv$num_amostras
    }
    posicao_ref <- posicao_max # posição superior onde o próximo conjunto de ICs será desenhado
    numerador <- 1   #  numerador da fração de passo que deverá ser reduzida de posicao_max
    denominador <- 2 #  denominador da fração de passo que deverá ser reduzida de posicao_max
    conjunto <- 1   # indica a amostragem cujo conjunto de ICs será desenhado
    contador <- 1   # contador do número de ICs que já foram desenhados no conjunto atual
    for (i in 1:rv$cont_conj_amostras) {
        if (rv$num_amostras == 1) {
            amostra <- rv$dados[, i]
            # para cada amostra, obtém a média e os limites do intervalo de confiança
            media_amostral <- mean(amostra)
            limInfIC <- media_amostral + qnorm(metade_alfa, mean = 0, sd = 1, lower.tail = TRUE) * erro_padrao
            limSupIC <- media_amostral + qnorm(metade_alfa, mean = 0, sd = 1, lower.tail = FALSE) * erro_padrao
            color <- "black"
            if (input$linha_ref == TRUE) {
                # se o intervalo não contiver a média, ele será plotado em vermelho
                # se contiver a média, será plotado em preto
                abline(v = rv$media_pop, col = "black", lwd = 2)
                color <-
                    if ((rv$media_pop < limInfIC) | (rv$media_pop > limSupIC)) {
                        "red"
                    } else {
                        "black"
                    }
            }
            posicao <- posicao_ref - (contador - 1) * passo # posição onde o IC será desenhado
            # traça o intervalo de confiança
            segments(limInfIC, posicao, limSupIC, posicao, col = color, lwd = 2)
            # traça um ponto para indicar a média da amostra
            points(media_amostral, posicao, col = "black", lwd = 2)
            contador <- contador + 1
            if (posicao < 20) {
                contador <- 1  # contador volta a 1 ao final de cada conjunto para o próximo IC ser traçado na parte de cima
                denominador <- 2^(trunc(log2(conjunto))+1)
                numerador = ((conjunto+1)- 2^(trunc(log2(conjunto)) )) * 2 - 1
                conjunto <- conjunto + 1  # próximo conjunto de ICs de uma amostra
                posicao_ref <- posicao_max - passo * numerador / denominador # posição do próximo conjunto de ICs de uma amostra  
            }
        } else {
            final <- i * rv$num_amostras  # posição final do próximo conjunto de num_amostras
            inicio <- 1 + (i - 1) * rv$num_amostras # posição inicial do próximo conjunto de num_amostras
            for (j in inicio:final) {
                amostra_j <- rv$dados[, j]
                # para cada amostra, obtém a média e os limites do intervalo de confiança
                media_amostral <- mean(amostra_j)
                limInfIC <- media_amostral + qnorm(metade_alfa, lower.tail = TRUE) * erro_padrao
                limSupIC <- media_amostral + qnorm(metade_alfa, lower.tail = FALSE) * erro_padrao
                color <- "black"
                if (input$linha_ref == TRUE) {
                    # se o intervalo não contiver a média, ele será plotado em vermelho
                    # se contiver a média, será plotado em preto
                    abline(v = rv$media_pop, col = "black", lwd = 2)
                    color <-
                        if ((rv$media_pop < limInfIC) | (rv$media_pop > limSupIC)) {
                            "red"
                        } else {
                            "black"
                        }
                }
                # traça o intervalo de confiança
                posicao <- posicao_ref - (j - inicio) * passo
                segments(limInfIC, posicao, limSupIC, posicao, col = color, lwd = 2)
                # traça um ponto para indicar a média da amostra
                points(media_amostral, posicao, col = color, lwd = 2)
            }
            posicao_ref <- posicao_max - passo * numerador / denominador
            numerador = ((i+2)- 2^(trunc(log2(i+1) )  )) * 2-1
            denominador = 2^(trunc(log2(i+1))+1)
        }
    }
  })
}

ui <- fluidPage(
  
  titlePanel(expression("Intervalos de confiança")),
  
  sidebarLayout(
    sidebarPanel(
      sliderInput("media", 
                label = expression("Média da população"), 
                min = 50, max = 100, step = 1,
                value = 80
    ),
    sliderInput("dp", 
                label = expression("Desvio padrão da população"), 
                min = 10, max = 50, step = 1,
                value = 20
    ),
    selectInput("confianca", 
                label = expression("Nível de confiança:"),
                choices = c("90%", "95%", "99%"),
                selected = "95%"),
    radioButtons("num_amostras", "Número de amostras:",
                 c("1" = 1,
                   "10" = 10,
                   "50" = 50,
                   "100" = 100), 
                 inline = TRUE, selected = 1),
    numericInput("n", 
                 label = "Tamanho da amostra", 
                 min = 1, max = 100, step = 1,
                 value = 10
    ),
    checkboxInput("linha_ref", "Mostrar média da população", value = FALSE),
    actionButton(inputId = "reamostrar", label = "Reamostrar"),
    actionButton(inputId = "reset", label = "Limpar")    
  ),
    
    mainPanel(
      plotOutput("ics", height=800))
  )
)

shinyApp(ui = ui, server = server)