#  
#   Teste de randomização e Intervalo de Confiança
#
#  Ilustra o conceito de teste de hipótese, valor de p e intervalo de confiança
#  baseado em testes de randomização  
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(stats)

server <- function(input, output) {

# 
  
x <- reactive({
  g1 = as.numeric(strsplit(input$valoresGrupo1, ",")[[1]])
  g2 = as.numeric(strsplit(input$valoresGrupo2, ",")[[1]])
  if ((mean(g1) - mean(g2)) < 0) {
    list(v = g2, grupo = input$grupo2)
  } else {
    list(v = g1, grupo = input$grupo1)
  }
})

y <- reactive({
  g1 = as.numeric(strsplit(input$valoresGrupo1, ",")[[1]])
  g2 = as.numeric(strsplit(input$valoresGrupo2, ",")[[1]])
  if ((mean(g1) - mean(g2)) < 0) {
    list(v = g1, grupo = input$grupo1)
  } else {
    list(v = g2, grupo = input$grupo2)
  }
})

variable <- reactive({
  input$variavel
})

parametros <- reactive({
  list(n1 = length(x()$v), total_sample = c(x()$v, y()$v), diff = mean(x()$v) - mean(y()$v),
       sample_size = length(y()$v) + length(x()$v)
  )
})

alfa_2 <- reactive({
  (100 - as.numeric(substring(input$confianca, 0, 2))) / 200
})

observeEvent(input$descritiva, {
  output$graficoDescritivo <- renderPlot({
    stripchart(list(x = x()$v, y = y()$v), vertical=TRUE, group.names =  c(x()$grupo, y()$grupo), method="stack", ylab=variable())
  })
})       


observeEvent(input$teste_hipotese, {
  output$histTesteHipotese <- renderPlot({
    total_sample = parametros()$total_sample
    n1 = parametros()$n1
    sample_size = parametros()$sample_size
    diff = parametros()$diff
    n = 100000
    diferenca = vector("numeric", n)
    diferenca[n] = diff
    m = n-1
    for (i in 1:m) {
      sample = sample(total_sample, sample_size)
      x_sample = sample[1:n1]
      first_y = n1 + 1
      y_sample = sample[(n1+1):sample_size]
      diferenca[i] = mean(x_sample) - mean(y_sample)
    }
    h = hist(diferenca)
    indice_diff = min(which(h$breaks > diff))
    cuts <- cut(h$breaks, c(-Inf, diff, Inf))
    plot(h, col=c("white", "red")[cuts])
    polygon(c(diff, diff, h$breaks[indice_diff], h$breaks[indice_diff]), c(0, h$counts[indice_diff-1], h$counts[indice_diff-1], 0), col="red")
    abline(v=diff)
    p = sum(diferenca >= diff)/n
  })
})       

observeEvent(input$ic_limInferior, {
  output$histIcInferior <- renderPlot({
    total_sample = parametros()$total_sample
    sample_size = parametros()$sample_size
    n1 = parametros()$n1
    diff = mean(x()$v) - mean(y()$v)

    n = 100000
    diferenca = vector("numeric", n)
    diferenca[n] = diff
    m = n-1
    for (i in 1:m) {
      sample = sample(total_sample, sample_size)
      x_sample = sample[1:n1]
      first_y = n1 + 1
      y_sample = sample[(n1+1):sample_size]
      diferenca[i] = mean(x_sample) - mean(y_sample)
    }
    h = hist(diferenca)
    indice_diff = min(which(h$breaks > diff))
    cuts <- cut(h$breaks, c(-Inf, diff, Inf))
    plot(h, col=c("white", "red")[cuts])
    polygon(c(diff, diff, h$breaks[indice_diff], h$breaks[indice_diff]), c(0, h$counts[indice_diff-1], h$counts[indice_diff-1], 0), col="red")
    abline(v=diff)
    p = sum(diferenca >= diff)/n    
    
    n = 50000
    m = n - 1
    diferenca = vector("numeric", n)
    # limite inferior
    if (p < alfa_2()) {
      lim_sup = diff
      current = diff / 2
      lim_inf = 0
    } else {
      lim_sup = 0
      lim_inf = 0 - 2 * diff
      current = -diff
    }
    p_lower = p
    while (abs(p_lower - alfa_2) > 0.003) {
      x_test = x()$v - current
      total_sample = c(x_test, y()$v)
      diferenca[n] = mean(x_test) - mean(y()$v)
      for (i in 1:m) {
        sample = sample(total_sample, sample_size)
        x_sample = sample[1:n1]
        y_sample = sample[(n1+1):sample_size]
        diferenca[i] = mean(x_sample) - mean(y_sample)
      }
      p_lower = sum((diferenca + current) >= diff) / n
      p_lower
      if (p_lower < alfa_2()) {
        lim_inf = current
        current = (lim_sup + current) / 2
      }  else {
        lim_sup = current
        current = (lim_inf + current) / 2
      }
    }
    lower_lim = current
    h = hist(diferenca + current)
    indice_diff = min(which(h$breaks > diff))
    cuts <- cut(h$breaks, c(-Inf, diff, Inf))
    plot(h, col = c("white", "red")[cuts])
    polygon(c(diff, diff, h$breaks[indice_diff], h$breaks[indice_diff]),
            c(0, h$counts[indice_diff - 1], h$counts[indice_diff - 1], 0),
            col = "red")
    abline(v = diff)
  })
})       

observeEvent(input$ic_limSuperior, {
  output$histIcSuperior <- renderPlot({
    total_sample = parametros()$total_sample
    sample_size = parametros()$sample_size
    n1 = parametros()$n1
    diff = mean(x()$v) - mean(y()$v)

    n = 100000
    diferenca = vector("numeric", n)
    diferenca[n] = diff
    m = n-1
    for (i in 1:m) {
      sample = sample(total_sample, sample_size)
      x_sample = sample[1:n1]
      first_y = n1 + 1
      y_sample = sample[(n1+1):sample_size]
      diferenca[i] = mean(x_sample) - mean(y_sample)
    }
    h = hist(diferenca)
    indice_diff = min(which(h$breaks > diff))
    cuts <- cut(h$breaks, c(-Inf, diff, Inf))
    plot(h, col=c("white", "red")[cuts])
    polygon(c(diff, diff, h$breaks[indice_diff], h$breaks[indice_diff]), c(0, h$counts[indice_diff-1], h$counts[indice_diff-1], 0), col="red")
    abline(v=diff)
    p = sum(diferenca >= diff)/n
    
    
        lim_sup = diff*3
      lim_inf = diff
      current = diff*2
    p_upper = p
    while (abs(p_upper - alfa_2()) > 0.003) {
      x_test = x()$v - current
      total_sample = c(x_test, y)
      diferenca[n] = mean(x_test) - mean(y()$v)
      for (i in 1:m) {
        sample = sample(total_sample, sample_size)
        x_sample = sample[1:n1]
        y_sample = sample[(n1+1):sample_size]
        diferenca[i] = mean(x_sample) - mean(y_sample)
      }
      p_upper = sum((diferenca + current) <= diff) / n
      p_upper
      if (p_upper < alfa_2()) {
        lim_sup = current
        current = (lim_inf + current) / 2
      }  else {
        lim_inf = current
        current = (lim_sup + current) / 2
      }
    }
    upper_lim = current
    h = hist(diferenca + current)
    indice_diff = max(which(h$breaks < diff))
    cuts <- cut(h$breaks, c(-Inf, h$breaks[indice_diff - 1], Inf))
    plot(h, col = c("red", "white")[cuts])
    polygon(c(h$breaks[indice_diff], h$breaks[indice_diff], diff, diff),
            c(0, h$counts[indice_diff], h$counts[indice_diff], 0),
            col = "red")
    abline(v = diff)
  })
})       

observeEvent(input$reset, {
  output$graficoDescritivo <- NULL
  output$histTesteHipotese <- NULL
  output$histIcInferior <- NULL
  output$histIcSuperior <- NULL
})       

}

ui <- fluidPage(
  
  titlePanel(expression("Testes de Hipóteses")),
  
  sidebarLayout(
    sidebarPanel(
      textInput("variavel", label = expression(Variável), 
                value = "Folato"
      ),
      textInput("grupo1", label = "Grupo 1", 
                value = "N2O+O2,24h"
      ),
      textInput("grupo2", label = "Grupo 2", 
                value = "N2O+O2,op"
      ),
      textInput("valoresGrupo1", label = "Valores no Grupo 1", 
                value = "110, 111, 107, 108, 110, 105, 107, 106, 111, 111"
      ),
      textInput("valoresGrupo2", label = "Valores no Grupo 2", 
                value = "120, 107, 110, 116, 114, 111, 113, 117, 114, 112"
      ),
      selectInput("confianca", 
                  label = expression("Nível de confiança:"),
                  choices = c("90%", "95%", "99%"),
                  selected = "95%"),
      actionButton(inputId = "descritiva", label = "Mostrar um diagrama Stripchart"),
    actionButton(inputId = "teste_hipotese", label = "Testar hipótese nula"),
    actionButton(inputId = "ic_limInferior", label = "Limite inferior do Intervalo de Confiança"),
    actionButton(inputId = "ic_limSuperior", label = "Limite superior do Intervalo de Confiança"),
    actionButton(inputId = "reset", label = "Limpar")    
  ),
    
    mainPanel(
      fluidRow(
        column(5, plotOutput("graficoDescritivo")),
        column(5, plotOutput("histTesteHipotese"))
      ),
      fluidRow(
        column(5, plotOutput("histIcInferior")),
        column(5, plotOutput("histIcSuperior"))
      )
    )
  )
)

shinyApp(ui = ui, server = server)