#
# aplicação que mostra o teorema do limite central para algumas distribuições 
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(RcmdrMisc)

server <- function(input, output, session) {
  
  # Distribuição normal
  dados <- reactive ({
      switch(input$dist, 
             "rnorm" = {
                 if (is.null(input$mu)) {
                     rnorm(input$n * input$num_amostras, mean=1, sd=1)
                 } else {  
                    rnorm(input$n * input$num_amostras, mean=input$mu, sd=input$dp)
                 }
            },
             "runif" = {
                 if (is.null(input$minmax[1])) {
                     runif(input$n * input$num_amostras, 5, 15)
                 }  else {  
                    runif(input$n * input$num_amostras, input$minmax[1], input$minmax[2])
                 }
            },
            "rgama" = {
                if (is.null(input$r)) {
                    rgamma(input$n * input$num_amostras, shape=2, rate = 1)
                } else { 
                    rgamma(input$n * input$num_amostras, shape=input$r, rate=input$alfa)
                }
           }
    )
  })

  output$mu = renderUI(
      {
          if (input$dist == "rnorm")
          {
              sliderInput("mu",
                          expression("Média"),
                          value = 1,
                          min = 0,
                          max = 50)
          }
      })

  # slider para o desvio padrão da distribuição Normal
  output$desvio_padrao = renderUI(
      {
          if (input$dist == "rnorm")
          {
              sliderInput("dp", 
                          expression("Desvio padrão"), 
                          min = 0, max = 50,
                          value = 1
              )
          }
      })
  
  output$alfag = renderUI(
      {
          if (input$dist == "rgama")
          {
              sliderInput("alfa", 
                          "Alfa", 
                          min = 0.1, max = 10,
                          value = 1)
          }
     })

  output$rg = renderUI(
      {
          if (input$dist == "rgama")
          {
              sliderInput("r", 
                          "r", 
                          min = 0.1, max = 10,
                          value = 2)
          }
      })
  
  # slider Minmax para a distribuição Uniforme  ----
  output$minmax = renderUI(
      {
          if (input$dist == "runif")
          {
              sliderInput("minmax",
                          "Limites Inferior e Superior:",
                          value = c(5, 15),
                          min = 0,
                          max = 20)
          }
      })
  
  # Se certificando que a amplitude da distribuição uniforme != 0 ----
  observeEvent(input$minmax, {
      req(input$minmax)
      if (input$minmax[1] == input$minmax[2]){
          if (input$minmax[1] == 0){
              updateSliderInput(session, "minmax", value = c(0, 1))
          } else if (input$minmax[2] == 20){
              updateSliderInput(session, "minmax", value = c(19, 20))
          } else {
              updateSliderInput(session, "minmax", value = c(input$minmax[2], input$minmax[2] + 1))
          }
      }
  })


  # traça o gráfico da distribuição selecionada
  output$grafico_dist <- renderPlot({
      switch(
          input$dist,
          "rnorm" = {
              if (is.null(input$mu)) {
                  mu = 1
                  dp = 1
              } else {
                  mu = input$mu
                  dp = input$dp
              }   
              lim_inf <- mu - 4 * dp
              lim_sup <- mu + 4 * dp
              curve(dnorm(x, mu, dp), xlim = c(lim_inf, lim_sup),
                  main = paste("N(", sprintf("%.1f", mu), ", ", sprintf("%.1f", dp ^ 2), ")", sep = ""),
                  xlab = "", ylab = "")
          },
          "runif" = {
              if (is.null(input$minmax[1])) {
                  minimo = 5
                  maximo = 15
              } else {
                  minimo = input$minmax[1]
                  maximo = input$minmax[2]
              }   
              curve(dunif(x, minimo, maximo),
                    xlim = c(minimo - 0.5, maximo + 0.5),
                    main = paste(expression("Distribuição Uniforme ("), sprintf("%.1f", minimo), ", ",
                                 sprintf("%.1f", maximo), ")", sep = ""), xlab = "", ylab = "")
                    ypos <- par("usr")[4]/2 # obtém a metade do valor máximo do eixo y
                    media <- (minimo + maximo) /2
                    xpos = media
                    variancia = (maximo-minimo)^2/12
                    text(xpos, ypos, paste(expression("Média = "), sprintf("%.2f", media)), pos = 4)
                    text(xpos, 3*ypos/4, paste(expression("Variância = "), sprintf("%.4f", variancia)), pos = 4)
          },
          "rgama" = {
              if (is.null(input$alfa)) {
                  alfa = 5
                  r = 15
              } else {
                  alfa = input$alfa
                  r = input$r
              }   
              curve(dgamma(x, r, alfa), xlim = c(-0.5, r * 3),
                  main = paste(expression("Distribuição Gamma ("), sprintf("%.1f", alfa), ", ", sprintf("%.1f", r), ")", sep = ""),
                  xlab = "", ylab = ""
              )
              ypos <- par("usr")[4]/2 # obtém a metade do valor máximo do eixo y
              media = r/alfa
              variancia = r/alfa^2
              xpos = media
              text(xpos, ypos, paste(expression("Média = "), sprintf("%.2f", media)), pos = 4)
              text(xpos, 3*ypos/4, paste(expression("Variância = "), sprintf("%.4f", variancia)), pos = 4)
          }
      )
  })
  
  # plota o histograma das médias amostrais da distribuição selecionada
  output$histDensidadeMediaAmostra <- renderPlot({
      # y é uma lista com três elementos:  
      #   uma matriz de duas dimensões (tamanho da amostral, número de amostras)
      #   valor mínimo do eixo x
      #   valor máximo do eixo x
      y <- switch(input$dist, 
           "rnorm" = {
               if (is.null(input$mu)) {
                   mu = 1
                   dp = 1
               } else {
                   mu = input$mu
                   dp = input$dp
               }   
             amostras <- matrix(dados(), nc = input$num_amostras)
             min_xaxis <- mu - 4 * dp
             max_xaxis <- mu + 4 * dp
             list(amostras, min_xaxis, max_xaxis)
           },
           "runif" = {
               if (is.null(input$minmax[1])) {
                   minimo = 5
                   maximo = 15
               } else {
                   minimo = input$minmax[1]
                   maximo = input$minmax[2]
               }   
               amostras <- matrix(dados(), nc = input$num_amostras)
               min_xaxis <- minimo - 0.5
               max_xaxis <- maximo + 0.5
               list(amostras, min_xaxis, max_xaxis)
           },
           "rgama" = {
               if (is.null(input$alfa)) {
                   alfa = 5
                   r = 15
               } else {
                   alfa = input$alfa
                   r = input$r
               }   
               amostras <- matrix(dados(), nc = input$num_amostras)
               min_xaxis <- -0.5
               max_xaxis <-  r * 3          
               list(amostras, min_xaxis, max_xaxis)
           })
    # ybar é um vetor que contém a média de cada amostra da matriz y
    ybar <- apply(y[[1]], 2, mean)
    # media_amostras é a média das médias amostrais
    media_amostras <- mean(ybar)
    # variancia é a variância das médias amostrais
    variancia <- var(ybar)
    min_xaxis <- y[[2]]
    max_xaxis <- y[[3]]
    par(cex=1.3)
    hist(ybar, freq=FALSE, breaks = input$num_classes, xlab = "", ylab=expression("Densidade de Frequência"), main ="", xlim=c(min_xaxis, max_xaxis))
    ypos <- par("usr")[4]/2 # obtém a metade do valor máximo do eixo y
    xpos <- min_xaxis + 3*(max_xaxis - min_xaxis)/4
    text(xpos, ypos, paste(expression("Média = "), sprintf("%.2f", media_amostras)), pos = 4)
    text(xpos, 3*ypos/4, paste(expression("Variância = "), sprintf("%.4f", variancia)), pos = 4)
  })
}

ui <- fluidPage(
  
  titlePanel("Teorema do Limite Central"),
  
  sidebarLayout(
    sidebarPanel(style = "font-size: 20px; ",
                 tags$style("[type = 'number'] {font-size:20px;height:50px;}"),
        radioButtons("dist", expression("Distribuição:"),
                     c("Normal" = "rnorm",
                       "Uniforme" = "runif",
                       "Gama" = "rgama"),
                     selected = "rnorm"),
        
      # parâmetros das distribuições:
      uiOutput("mu"),
      uiOutput("desvio_padrao"),
      uiOutput("alfag"),
      uiOutput("rg"),
      uiOutput("minmax"),
      
      sliderInput("num_amostras", 
                  expression("Número de amostras"), 
                  min = 1000, max = 100000,
                  value = 10000
      ),
      sliderInput("num_classes", 
                  expression("Número de classes no histograma:"), 
                  min = 2, max = 100,
                  value = 20
      ),
      sliderInput("n",
                  expression("Tamanho amostral"),
                  value = 1,
                  min = 1,
                  max = 200) 
    ),
    
    mainPanel(
      fluidRow(
        column(8, plotOutput("grafico_dist")),
        column(8, plotOutput("histDensidadeMediaAmostra"))
      )
    )
  )
)

shinyApp(ui = ui, server = server)